#!/bin/bash

AWK="/usr/bin/awk"
ECHO="/bin/echo"
FIND="/usr/bin/find"
GREP="/usr/bin/grep"
ID="/usr/xpg4/bin/id"
LS="/bin/ls"
PKGCHK="/usr/sbin/pkgchk"
SED="/usr/bin/sed"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
UNAME="/bin/uname"
WC="/bin/wc"
XARGS="/usr/bin/xargs"

OS="unknown"
OS_VERSION="unknown"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_daemon_umask {
   print_banner "Checking Daemon umask"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${GREP} "^CMASK=022" /etc/default/init >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "CMASK not set correctly in /etc/default/init"
             print_info "It is recommended that you restrict your daemon processes so that they"
             print_info "cannot create world writable files. Execute the following commands to"
             print_info "implement this restriction:"
             print_info "cd /etc/default"
             print_info "sed 's/^[#]\{0,1\}CMASK=.*$/CMASK=022/' init > init.new"
             print_info "mv init.new init"
             print_info "pkgchk -f -n -p /etc/default/init"
          else
             print_ok "CMASK set correctly in /etc/default/init"
          fi 
          ;;
   esac
}

function check_rmmount_conf {
   print_banner "Checking rmmount.conf"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${GREP} "^mount" /etc/rmmount.conf | while read MOUNT_LINE; do
             ${ECHO} "${MOUNT_LINE}" | ${GREP} -- "-o nosuid" >/dev/null 2>&1
             if [ "$?" -ne "0" ]; then
                print_warn "Line: \"${MOUNT_LINE}\" in /etc/rmmount.conf does not have nosuid option set"
             else
                print_ok "Line: \"${MOUNT_LINE}\" in /etc/rmmount.conf has nosuid option set"
             fi
          done 
          ;;
   esac
}

function check_passwd_files {
   print_banner "Checking passwd Files"
   print_divider
   case ${OS_VERSION} in
      "10")
          PASSWD_FILES="/etc/passwd /etc/group /etc/shadow"
          print_info "/etc/passwd, /etc/group and /etc/shadow will now be checked with pkgchk."
          print_info "Review the output (if any) and take corrective action as required"
          for PASSWD_FILE in ${PASSWD_FILES}; do
             ${PKGCHK} -p ${PASSWD_FILE}    
          done
          ;;
   esac 
}

function check_world_writable_directories {
   print_banner "Checking World Writable Directories For Sticky Bit"
   print_divider
   case ${OS_VERSION} in
      "10")
          print_info "Preparing a list of directories that are world-writable and DO NOT"
          print_info "have their sticky bit set - review and correct if necessary"
          ${FIND} / \( -fstype nfs -o -fstype cachefs \) -prune -o \
                  -type d \( -perm -0002 -a ! -perm -1000 \) -exec ${LS} -ld {} \; 2>/dev/null
          ;;
   esac
}

function check_world_writable_files {
   print_banner "Checking for World Writable Files"
   print_divider
   case ${OS_VERSION} in
      "10")
          print_info "Preparing a list of files that are world-writable review and correct if necessary"
          ${FIND} / \( -fstype nfs -o -fstype cachefs \) -prune -o \
                  -type f -perm -0002 -exec ${LS} -ld {} \; 2>/dev/null
          ;;
   esac
}

function check_suid_sgid_executables {
   print_banner "Checking for SUID/SGID System Executables"
   print_divider
   case ${OS_VERSION} in
      "10")
          print_info "Preparing a list of files that are SUID/SGID and executable"
          ${FIND} / \( -fstype nfs -o -fstype cachefs \) -prune -o \
                  -type f \( -perm -04001 -o -perm -02010 \) -exec ${LS} -ld {} \; 2>/dev/null
          ;;
   esac
}

function check_unowned_files_and_directories {
   print_banner "Checking for Unowned Files and Directories"
   print_divider
   case ${OS_VERSION} in
     "10")
          print_info "Checking for \"unowned\" files and directories"
          ${FIND} / \( -fstype nfs -o -fstype cachefs \) -prune -o \
                  \( -nouser -o -nogroup \) -exec ${LS} -ld {} \; 2>/dev/null
         ;;
   esac
}

function check_extended_attributes {
   print_banner "Checking for Extended Attributes"
   print_divider
   case ${OS_VERSION} in
     "10")
          print_info "Checking for files and directories with exteneded attributes"
          ${FIND} / \( -fstype nfs -o -fstype cachefs \) -prune -o \
                  -xattr -exec ${LS} -ld {} \; 2>/dev/null
         ;;
   esac
}

check_os
check_user
print_divider
check_daemon_umask
print_divider
check_rmmount_conf
print_divider
check_passwd_files
print_divider
check_world_writable_directories
print_divider
check_world_writable_files
print_divider
check_suid_sgid_executables
print_divider
check_unowned_files_and_directories
print_divider
check_extended_attributes

exit 0
